from typing import Optional

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc


@abaqus_class_doc
class ShearTestData:
    r"""The ShearTestData object specifies the normalized shear creep compliance or relaxation modulus as a
    function of time.

    .. note::
        This object can be accessed by::

            import material
            mdb.models[name].materials[name].viscoelastic.shearTestData
            import odbMaterial
            session.odbs[name].materials[name].viscoelastic.shearTestData

        The table data for this object are:

        - If **time** = RELAXATION_TEST_DATA, the table data specify the following:

            - Normalized shear relaxation modulus :math:`g_{R}(t)`. :math:`\left(0 \leq g_{R}(t) \leq 1\right)`
            - Time :math:`t (t>0)`.

        - If **time** = CREEP_TEST_DATA, the table data specify the following:

            - Normalized shear compliance :math:`j_{S}(t)`. :math:`\left(j_{S}(t) \geq 1\right)`.
            - Time :math:`t (t>0)`.

        The corresponding analysis keywords are:

        - SHEAR TEST DATA
    """

    @abaqus_method_doc
    def __init__(self, table: tuple, shrinf: Optional[float] = None):
        r"""This method creates a ShearTestData object.

        .. note::
            This function can be accessed by::

                mdb.models[name].materials[name].viscoelastic.ShearTestData
                session.odbs[name].materials[name].viscoelastic.ShearTestData

        Parameters
        ----------
        table
            A sequence of sequences of Floats specifying values that depend on the **time** member of
            the Viscoelastic object.
        shrinf
            None or a Float specifying a normalized shear. The value of **shrinf** depends on the
            value of the **time** member of the Viscoelastic object. The default value is None.If
            **time** = RELAXATION_TEST_DATA, **shrinf** specifies the value of the long-term, normalized
            shear modulus :math:`g_R(\infty)`.If **time** = CREEP_TEST_DATA, **shrinf** specifies the value of the
            long-term, normalized shear compliance :math:`j_S(\infty)`.

        Returns
        -------
        ShearTestData
            A ShearTestData object.
        """
        ...

    @abaqus_method_doc
    def setValues(self, *args, **kwargs):
        """This method modifies the ShearTestData object."""
        ...



from typing import Optional

from abqpy.decorators import abaqus_class_doc

from ..UtilityAndView.abaqusConstants import SymbolicConstant
from .BoundaryConditionState import BoundaryConditionState


@abaqus_class_doc
class SecondaryBaseBCState(BoundaryConditionState):
    """The SecondaryBaseBCState object stores the propagating data for a slave base boundary condition in a
    step. One instance of this object is created internally by the SecondaryBaseBC object for each step. The
    instance is also deleted internally by the SecondaryBaseBC object. The SecondaryBaseBCState object has no
    constructor or methods. The SecondaryBaseBCState object is derived from the BoundaryConditionState object.

    .. note::
        This object can be accessed by::

            import load
            mdb.models[name].steps[name].boundaryConditionStates[name]

        The corresponding analysis keywords are:

        - BOUNDARY
    """

    #: A SymbolicConstant specifying the propagation state of the amplitude reference. Possible
    #: values are UNSET, SET, UNCHANGED, FREED, and MODIFIED.
    amplitudeState: Optional[SymbolicConstant] = None

    #: A SymbolicConstant specifying the propagation state of the BoundaryConditionState object. Possible values are:
    #:
    #: - NOT_YET_ACTIVE
    #: - CREATED
    #: - PROPAGATED
    #: - MODIFIED
    #: - DEACTIVATED
    #: - NO_LONGER_ACTIVE
    #: - TYPE_NOT_APPLICABLE
    #: - INSTANCE_NOT_APPLICABLE
    #: - PROPAGATED_FROM_BASE_STATE
    #: - MODIFIED_FROM_BASE_STATE
    #: - DEACTIVATED_FROM_BASE_STATE
    #: - BUILT_INTO_MODES
    status: Optional[SymbolicConstant] = None

    #: A String specifying the name of the amplitude reference. The String is empty if the
    #: boundary condition has no amplitude reference.
    amplitude: str = ""



from typing import Dict

from typing_extensions import Literal

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ..Amplitude.Amplitude import Amplitude
from ..BeamSectionProfile.Profile import Profile
from ..CustomKernel.RepositorySupport import RepositorySupport
from ..Filter.Filter import Filter
from ..Material.Material import Material
from ..Section.Section import Section
from ..UtilityAndView.abaqusConstants import CLOSEST, OFF, Boolean
from ..UtilityAndView.abaqusConstants import abaqusConstants as C
from .JobData import JobData
from .OdbAssembly import OdbAssembly
from .OdbPart import OdbPart
from .OdbStep import OdbStep
from .SectionCategory import SectionCategory
from .SectorDefinition import SectorDefinition
from .UserData import UserData


@abaqus_class_doc
class OdbBase:
    """The Odb object is the in-memory representation of an output database (ODB) file.

    .. note::
        This object can be accessed by::

            import odbAccess
            session.odbs[name]
    """

    #: A Boolean specifying whether the output database was opened with read-only access.
    isReadOnly: Boolean = OFF

    #: A repository of Amplitude objects.
    amplitudes: Dict[str, Amplitude] = {}

    #: A repository of Filter objects.
    filters: Dict[str, Filter] = {}

    #: An OdbAssembly object.
    rootAssembly: OdbAssembly = OdbAssembly()

    #: A JobData object.
    jobData: JobData = JobData()

    #: A repository of OdbPart objects.
    parts: Dict[str, OdbPart] = {}

    #: A repository of Material objects.
    materials: Dict[str, Material] = {}

    #: A repository of OdbStep objects.
    steps: Dict[str, OdbStep] = {}

    #: A repository of Section objects.
    sections: Dict[str, Section] = {}

    #: A repository of SectionCategory objects.
    sectionCategories: Dict[str, SectionCategory] = {}

    #: A SectorDefinition object.
    sectorDefinition: SectorDefinition = SectorDefinition()

    #: A UserData object.
    userData: UserData = UserData()

    #: A RepositorySupport object.
    customData: RepositorySupport = RepositorySupport()

    #: A repository of Profile objects.
    profiles: Dict[str, Profile] = {}

    @abaqus_method_doc
    def __init__(self, name: str, analysisTitle: str = "", description: str = "", path: str = ""):
        """This method creates a new Odb object.

        .. note::
            This function can be accessed by::

                session.Odb

        Parameters
        ----------
        name
            A String specifying the repository key.
        analysisTitle
            A String specifying the title of the output database. The default value is an empty
            string.
        description
            A String specifying the description of the output database. The default value is an
            empty string.
        path
            A String specifying the path to the file where the new output database (.odb ) file will
            be written. The default value is an empty string.

        Returns
        -------
        Odb
            An Odb object.
        """
        ...

    @abaqus_method_doc
    def close(self):
        """This method closes an output database."""
        ...

    @abaqus_method_doc
    def getFrame(self, frameValue: str, match: Literal[C.BEFORE, C.EXACT, C.AFTER, C.CLOSEST] = CLOSEST):
        """This method returns the frame at the specified time, frequency, or mode. It will not interpolate
        values between frames. The method is not applicable to an Odb object containing steps with different
        domains or to an Odb object containing a step with load case specific data.

        Parameters
        ----------
        frameValue
            A Double specifying the value at which the frame is required. **frameValue** can be the
            total time or frequency.
        match
            A SymbolicConstant specifying which frame to return if there is no frame at the exact
            frame value. Possible values are CLOSEST, BEFORE, AFTER, and EXACT. The default value is
            CLOSEST.When **match** = CLOSEST, Abaqus returns the closest frame. If the frame value
            requested is exactly halfway between two frames, Abaqus returns the frame after the
            value.When **match** = EXACT, Abaqus raises an exception if the exact frame value does not
            exist.

        Returns
        -------
        OdbFrame
            An OdbFrame object.

        Raises
        ------
        OdbError
            Frame not found, If the exact frame is not found.
        """
        ...

    @abaqus_method_doc
    def save(self):
        """This method saves output to an output database (.odb ) file.

        Raises
        ------
        OdbError
            Database save failed. The database was opened as read-only. Modification of data is
            not permitted.
        """
        ...

    @abaqus_method_doc
    def update(self):
        """This method is used to update an Odb object in memory while an Abaqus analysis writes data to the
        associated output database. update checks if additional steps have been written to the output database
        since it was opened or last updated. If additional steps have been written to the output database,
        update adds them to the Odb object.

        Returns
        -------
        Boolean
            A Boolean specifying whether additional steps or frames were added to the Odb object.
        """
        ...



from typing import Sequence

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from .ConstrainedSketchGeometry import ConstrainedSketchGeometry


@abaqus_class_doc
class Line(ConstrainedSketchGeometry):
    @abaqus_method_doc
    def __init__(self, point1: Sequence[float], point2: Sequence[float]):
        """This method creates a line between two given points.

        .. note::
            This function can be accessed by::

                mdb.models[name].sketches[name].Line

        Parameters
        ----------
        point1
            A pair of Floats specifying the first endpoint.
        point2
            A pair of Floats specifying the second endpoint.

        Returns
        -------
        ConstrainedSketchGeometry
            A ConstrainedSketchGeometry object (None if the line cannot be created).
        """
        ...



from typing import Optional

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ..UtilityAndView.abaqusConstants import ON, Boolean
from .Area import Area
from .AreaStyle import AreaStyle
from .Axis import Axis
from .Legend import Legend
from .LineStyle import LineStyle
from .TextStyle import TextStyle


@abaqus_class_doc
class DefaultChartOptions:
    """The DefaultChartOptions object is used to hold on default chart and axis attributes. The
    DefaultChartOptions object attributes are used whenever Chart or Axis are created. A DefaultChartOptions
    object is automatically created when opening a session.

    .. note::
        This object can be accessed by::

            import visualization
            session.defaultChartOptions
    """

    areaStyle: AreaStyle = AreaStyle()

    @abaqus_method_doc
    def setValues(
        self,
        areaStyle: AreaStyle,
        aspectRatio: Optional[float] = None,
        defaultAxis1Options: Optional[Axis] = None,
        defaultAxis2Options: Optional[Axis] = None,
        gridArea: Optional[Area] = None,
        legend: Optional[Legend] = None,
        majorAxis1GridStyle: Optional[LineStyle] = None,
        majorAxis2GridStyle: Optional[LineStyle] = None,
        minorAxis1GridStyle: Optional[LineStyle] = None,
        minorAxis2GridStyle: Optional[LineStyle] = None,
        tagAreaStyle: Optional[AreaStyle] = None,
        tagBorder: Optional[LineStyle] = None,
        tagTextStyle: Optional[TextStyle] = None,
        useQuantityType: Boolean = ON,
    ):
        """This method modifies the DefaultChartOptions object.

        Parameters
        ----------
        areaStyle
            An AreaStyle object specifying an AreaStyle used to hold on to the default display
            properties for the chart area.
        aspectRatio
            A Float specifying the default aspect ratio of the grid area. A value of -1 specifies
            that the gridArea will take up all available space. The default value is −1.
        defaultAxis1Options
            An Axis object specifying an Axis object used to hold on to the default properties for
            direction 1 axes—the abscissa for a Cartesian chart.
        defaultAxis2Options
            An Axis object specifying an Axis object used to hold on to the default properties for
            direction 2 axes—the ordinate for a Cartesian chart.
        gridArea
            An Area object specifying how to display the grid area by default.
        legend
            A Legend object specifying the default attributes for the legend of the chart.
        majorAxis1GridStyle
            A LineStyle object specifying the default line properties to be used when drawing major
            gridlines along axis 1.
        majorAxis2GridStyle
            A LineStyle object specifying the default line properties to be used when drawing major
            gridlines along axis 2.
        minorAxis1GridStyle
            A LineStyle object specifying the default line properties to be used when drawing minor
            gridlines along axis 1.
        minorAxis2GridStyle
            A LineStyle object specifying the default line properties to be used when drawing minor
            gridlines along axis 2.
        tagAreaStyle
            An AreaStyle object specifying the default area properties to be used when creating
            tags.
        tagBorder
            A LineStyle object specifying the default tag area border properties to be used when
            creating tags.
        tagTextStyle
            A TextStyle object specifying the default text properties to be used when creating tags.
        useQuantityType
            A Boolean specifying whether to use the QuantityType to associate curves with axes. The
            default value is ON.
        """
        ...



from typing_extensions import Literal

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ..UtilityAndView.abaqusConstants import abaqusConstants as C
from .Edge import Edge


@abaqus_class_doc
class InterestingPoint:
    """Interesting points can be located at the following:

    - The middle of an edge.
    - The middle of an arc.
    - The center of an arc.

    An InterestingPoint object is a temporary object and cannot be accessed from the Mdb
    object.

    .. note::
        This object can be accessed by::

            import part
            import assembly
    """

    @abaqus_method_doc
    def __init__(self, edge: Edge, rule: Literal[C.MIDDLE, C.CENTER]):
        """This method creates an interesting point along an edge. An InterestingPoint is a temporary object.

        .. note::
            This function can be accessed by::

                mdb.models[name].parts[name].InterestingPoint
                mdb.models[name].rootAssembly.instances[name].InterestingPoint

        Parameters
        ----------
        edge
            An Edge object specifying the edge on which the interesting point is positioned.
        rule
            A SymbolicConstant specifying the position of the interesting point. Possible values are
            MIDDLE or CENTER.

        Returns
        -------
        InterestingPoint
            An InterestingPoint object.
        """
        ...



from abaqus.DisplayGroup.Leaf import Leaf
from abaqus.DisplayGroup.LeafFromDatums import LeafFromDatums
from abaqus.DisplayGroup.LeafFromDisplayGroup import LeafFromDisplayGroup
from abaqus.DisplayGroup.LeafFromGeometry import LeafFromGeometry
from abaqus.DisplayGroup.LeafFromInstance import LeafFromInstance
from abaqus.DisplayGroup.LeafFromInstanceElementLabels import (
    LeafFromInstanceElementLabels,
)
from abaqus.DisplayGroup.LeafFromInstanceNodeLabels import LeafFromInstanceNodeLabels
from abaqus.DisplayGroup.LeafFromMeshElementLabels import LeafFromMeshElementLabels
from abaqus.DisplayGroup.LeafFromMeshNodeLabels import LeafFromMeshNodeLabels
from abaqus.DisplayGroup.LeafFromMeshSurfaceSets import LeafFromMeshSurfaceSets
from abaqus.DisplayGroup.LeafFromPartElementLabels import LeafFromPartElementLabels
from abaqus.DisplayGroup.LeafFromPartNodeLabels import LeafFromPartNodeLabels
from abaqus.DisplayGroup.LeafFromReferencePoint import LeafFromReferencePoint
from abaqus.DisplayGroup.LeafFromSets import LeafFromSets



from typing import Optional

from abqpy.decorators import abaqus_class_doc

from ..UtilityAndView.abaqusConstants import SymbolicConstant


@abaqus_class_doc
class InteractionState:
    """The InteractionState object is the abstract base type for other InteractionState objects. The
    InteractionState object has no explicit constructor. The members of the InteractionState object are common
    to all objects derived from InteractionState.

    .. note::
        This object can be accessed by::

            import interaction
            mdb.models[name].steps[name].interactionStates[name]
    """

    #: A SymbolicConstant specifying the propagation state of the InteractionState object.
    #: Possible values are:
    #:
    #: - NOT_YET_ACTIVE
    #: - CREATED
    #: - PROPAGATED
    #: - MODIFIED
    #: - DEACTIVATED
    #: - NO_LONGER_ACTIVE
    #: - TYPE_NOT_APPLICABLE
    #: - INSTANCE_NOT_APPLICABLE
    #: - BUILT_INTO_BASE_STATE
    status: Optional[SymbolicConstant] = None



from typing import Optional

from typing_extensions import Literal

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ..Region.Region import Region
from ..UtilityAndView.abaqusConstants import INTERNAL, UNIFORM, UNSET, SymbolicConstant
from ..UtilityAndView.abaqusConstants import abaqusConstants as C
from .Load import Load


@abaqus_class_doc
class PipePressure(Load):
    """The PipePressure object stores the data for a pressure applied to pipe or elbow elements. The
    PipePressure object is derived from the Load object.

    .. note::
        This object can be accessed by::

            import load
            mdb.models[name].loads[name]
    """

    #: A String specifying the load repository key.
    name: str = ""

    #: A SymbolicConstant specifying whether the load is uniform. Possible values are UNIFORM,
    #: HYDROSTATIC, USER_DEFINED, and FIELD. The default value is UNIFORM.
    distributionType: SymbolicConstant = UNIFORM

    #: A SymbolicConstant specifying whether the pressure is applied internally or externally.
    #: Possible values are INTERNAL and EXTERNAL. The default value is INTERNAL.
    side: SymbolicConstant = INTERNAL

    #: A Float specifying the effective inner or outer diameter.
    diameter: Optional[float] = None

    #: A String specifying the name of the AnalyticalField object associated with this load.
    #: The **field** argument applies only when **distributionType** = FIELD. The default value is an
    #: empty string.
    field: str = ""

    #: A Region object specifying the region to which the load is applied.
    region: Region = Region()

    @abaqus_method_doc
    def __init__(
        self,
        name: str,
        createStepName: str,
        region: Region,
        magnitude: float,
        diameter: float,
        hZero: float,
        hReference: float,
        field: str = "",
        amplitude: str = UNSET,
        distributionType: Literal[C.USER_DEFINED, C.HYDROSTATIC, C.UNIFORM, C.FIELD] = UNIFORM,
        side: Literal[C.EXTERNAL, C.INTERNAL] = INTERNAL,
    ):
        """This method creates a Pressure object.

        .. note::
            This function can be accessed by::

                mdb.models[name].PipePressure

        Parameters
        ----------
        name
            A String specifying the load repository key.
        createStepName
            A String specifying the name of the step in which the pressure is created.
        region
            A Region object specifying the region to which the load is applied.
        magnitude
            A Float specifying the pressure magnitude. Note: *magnitude* is optional if
            **distributionType** = USER_DEFINED.
        diameter
            A Float specifying the effective inner or outer diameter.
        hZero
            A Float specifying the height of the zero pressure level when
            **distributionType** = HYDROSTATIC.
        hReference
            A Float specifying the height of the reference pressure level when
            **distributionType** = HYDROSTATIC.
        field
            A String specifying the name of the AnalyticalField object associated with this load.
            The **field** argument applies only when **distributionType** = FIELD. The default value is an
            empty string.
        amplitude
            A String or the SymbolicConstant UNSET specifying the name of the amplitude reference.
            UNSET should be used if the load has no amplitude reference. The default value is UNSET.
            You should provide the **amplitude** argument only if it is valid for the specified step.
        distributionType
            A SymbolicConstant specifying whether the load is uniform. Possible values are UNIFORM,
            HYDROSTATIC, USER_DEFINED, and FIELD. The default value is UNIFORM.
        side
            A SymbolicConstant specifying whether the pressure is applied internally or externally.
            Possible values are INTERNAL and EXTERNAL. The default value is INTERNAL.

        Returns
        -------
        PipePressure
            A PipePressure object.
        """
        super().__init__()

    @abaqus_method_doc
    def setValues(
        self,
        field: str = "",
        amplitude: str = UNSET,
        distributionType: Literal[C.USER_DEFINED, C.HYDROSTATIC, C.UNIFORM, C.FIELD] = UNIFORM,
        side: Literal[C.EXTERNAL, C.INTERNAL] = INTERNAL,
    ):
        """This method modifies the data for an existing PipePressure object in the step where it is created.

        Parameters
        ----------
        field
            A String specifying the name of the AnalyticalField object associated with this load.
            The **field** argument applies only when **distributionType** = FIELD. The default value is an
            empty string.
        amplitude
            A String or the SymbolicConstant UNSET specifying the name of the amplitude reference.
            UNSET should be used if the load has no amplitude reference. The default value is UNSET.
            You should provide the **amplitude** argument only if it is valid for the specified step.
        distributionType
            A SymbolicConstant specifying whether the load is uniform. Possible values are UNIFORM,
            HYDROSTATIC, USER_DEFINED, and FIELD. The default value is UNIFORM.
        side
            A SymbolicConstant specifying whether the pressure is applied internally or externally.
            Possible values are INTERNAL and EXTERNAL. The default value is INTERNAL.
        """
        ...

    @abaqus_method_doc
    def setValuesInStep(
        self,
        stepName: str,
        magnitude: Optional[float] = None,
        hZero: Optional[float] = None,
        hReference: Optional[float] = None,
        amplitude: str = "",
    ):
        """This method modifies the propagating data for an existing PipePressure object in the specified step.

        Parameters
        ----------
        stepName
            A String specifying the name of the step in which the load is modified.
        magnitude
            A Float specifying the pressure magnitude.
        hZero
            A Float specifying the height of the zero pressure level when
            **distributionType** = HYDROSTATIC.
        hReference
            A Float specifying the height of the reference pressure level when
            **distributionType** = HYDROSTATIC.
        amplitude
            A String or a SymbolicConstant specifying the name of the amplitude reference. Possible
            values for the SymbolicConstant are UNCHANGED and FREED. UNCHANGED should be used if the
            amplitude is propagated from the previous static analysis step. FREED should be used if
            the load has no amplitude reference. You should provide the **amplitude** argument only if
            it is valid for the specified step.
        """
        ...



from typing import Optional

from abqpy.decorators import abaqus_class_doc

from ..UtilityAndView.abaqusConstants import SymbolicConstant


@abaqus_class_doc
class DataObject:
    """An instance of the DataObject object is passed to each callback.

    The DataObject object has no methods. The members of a DataObject object depend on the type of the
    object. All DataObject instances have the following members, regardless of type:
    """

    #: A SymbolicConstant specifying the phase of the analysis. Possible values are
    #: BATCHPRE_PHASE, PACKAGER_PHASE, STANDARD_PHASE, EXPLICIT_PHASE, CALCULATOR_PHASE, and
    #: UNKNOWN_PHASE.
    phase: Optional[SymbolicConstant] = None

    #: An Int specifying the process ID of the analysis product.
    processId: Optional[int] = None

    #: An Int specifying the thread ID of the analysis product. Threads are used for parallel
    #: or multiprocessing; in most cases **threadId** is set to zero.
    threadId: Optional[int] = None

    #: An Int specifying the time the message was sent in seconds since 00:00:00 UTC, January
    #: 1, 1970.
    timeStamp: Optional[int] = None

    #: An Int specifying the number of attempts made to reach equilibrium during this step.
    attempts: Optional[int] = None

    #: An Int specifying the degree of freedom requested for monitoring the output.
    dof: Optional[int] = None

    #: An Int specifying the number of equilibrium iterations made during this increment.
    equilibrium: Optional[int] = None

    #: An Int specifying the increment of the analysis.
    increment: Optional[int] = None

    #: An Int specifying the number of iterations in the step.
    iterations: Optional[int] = None

    #: An Int specifying the node number requested for monitoring output.
    node: Optional[int] = None

    #: An Int specifying the number of severe discontinuity iterations completed during this
    #: increment.
    severe: Optional[int] = None

    #: An Int specifying the current step number. Step number 1 corresponds to the first step.
    step: Optional[int] = None

    #: An Int specifying the ID of the step.
    stepId: Optional[int] = None

    #: A Float specifying the step time corresponding to the current increment.
    stepTime: Optional[float] = None

    #: A Float specifying the total time corresponding to the monitor data.
    time: Optional[float] = None

    #: A Float specifying the time increment used in the current step.
    timeIncrement: Optional[float] = None

    #: A Float specifying the total time completed in the analysis.
    totalTime: Optional[float] = None

    #: A Float specifying the current value of the degree of freedom requested for monitoring.
    value: Optional[float] = None

    #: A String specifying the host name of the machine that is running the analysis.
    clientHost: str = ""

    #: A String specifying the name of the client that responded to the callback function.
    #: Possible values are BatchPre, Packager, Standard, Explicit, and Calculator.
    clientName: str = ""

    #: A String specifying the full path of the output database.
    file: str = ""

    #: A String specifying the job heading.
    heading: str = ""

    #: A String specifying the job heading.
    message: str = ""

    #: A String specifying the node set specified for monitoring output.
    nset: str = ""

    #: A String specifying the name of the step.
    stepName: str = ""



from typing import Optional

from abqpy.decorators import abaqus_class_doc

from .Datum import Datum


@abaqus_class_doc
class DatumPoint(Datum):
    """The DatumPoint object has no direct constructor; it is created when a Feature object is created. For
    example, the DatumPointByCoordinate method creates a Feature object that creates a DatumPoint object. The
    DatumPoint object is derived from the Datum object.

    .. note::
        This object can be accessed by::

            import part
            mdb.models[name].parts[name].datums[i]
            import assembly
            mdb.models[name].rootAssembly.allInstances[name].datums[i]
            mdb.models[name].rootAssembly.datums[i]
            mdb.models[name].rootAssembly.instances[name].datums[i]
    """

    #: A tuple of Floats specifying the **X**, **Y**, and **Z** coordinates of a point located on
    #: the datum.
    pointOn: Optional[float] = None



from abaqus.Job.Job import Job
from abaqus.Odb.AnalyticSurfaceSegment import AnalyticSurfaceSegment
from abaqus.Odb.HistoryPoint import HistoryPoint
from odbAccess import (
    AnalyticSurfaceProfile,
    Odb,
    isUpgradeRequiredForOdb,
    openOdb,
    upgradeOdb,
)

# from odbAccess import closeOdb
# from odbAccess import truncateOdb



import os
import re

import fire
import requests


class ReadTheDocsAPI:
    """ReadTheDocs API client"""

    def __init__(self, project: str = "abqpy", token: str = None):
        self.project = project
        self.TOKEN = token or os.environ.get("RTD_TOKEN")
        self.HEADERS = {"Authorization": f"token {self.TOKEN}"}


class ReadTheDocsVersion(ReadTheDocsAPI):
    """Manage versions of a project"""

    def build(self, version: str):
        """Build a version

        Parameters
        ----------
        version : str
            Version to build
        """
        URL = f"https://readthedocs.org/api/v3/projects/{self.project}/versions/{version}/builds/"
        print(requests.post(URL, headers=self.HEADERS))

    def update(self, version: str, *, active: bool = None, hidden: bool = None, privacy_level: str = None):
        """Update status of a version

        Parameters
        ----------
        version : str
            Version to update
        active : bool, optional
            Whether the version is active
        hidden : bool, optional
            Whether the version is hidden
        privacy_level : str, optional
            Privacy level of the version
        """
        URL = f"https://readthedocs.org/api/v3/projects/{self.project}/versions/{version}/"
        data = {}
        for option in ("active", "hidden", "privacy_level"):
            if locals()[option] is not None:
                data[option] = locals()[option]
        print(requests.patch(URL, json=data, headers=self.HEADERS))


class ReadTheDocsRedirect(ReadTheDocsAPI):
    """Manage redirects of a project"""

    def _lists(self) -> requests.Response:
        """List redirects

        Returns
        -------
        Response from the API
        """
        URL = f"https://readthedocs.org/api/v3/projects/{self.project}/redirects/"
        return requests.get(URL, headers=self.HEADERS)

    def update(self, minor_patch: str):
        """Update a redirect from /{lang}/{major} to the latest /{lang}/{major}.{minor}.{patch} version

        Parameters
        ----------
        minor_patch : str
            {minor}.{patch} version to update
        """
        URL = f"https://readthedocs.org/api/v3/projects/{self.project}/redirects/{{pk}}/"
        for redirect in self._lists().json()["results"]:
            url = URL.format(pk=redirect["pk"])
            from_url, to_url = redirect["from_url"], redirect["to_url"]
            lang, pfx, major, minor, patch, sfx = re.match(r"/(\w+)?/([vV]?)(\d+)\.(\d+)\.(\d+)(.+?)", to_url).groups()
            data = dict(url=url, from_url=from_url, to_url=f"/{lang}/{pfx}{major}.{minor_patch}{sfx}", type="exact")
            print(requests.put(url, json=data, headers=self.HEADERS))


class ReadTheDocs(ReadTheDocsAPI):
    """Manage a project on ReadTheDocs"""

    def __init__(self, project: str = "abqpy", token: str = None):
        super().__init__(project, token)
        self.version = ReadTheDocsVersion(project, token)
        self.redirect = ReadTheDocsRedirect(project, token)


if __name__ == "__main__":
    # Print to stdout, a workaround from https://github.com/google/python-fire/issues/188#issuecomment-1528976874
    fire.core.Display = lambda lines, out: out.write("\n".join(lines) + "\n")
    fire.Fire(ReadTheDocs)



from regionToolset import Region
from sketch import Sketch, SketchTransform
from xyPlot import (
    AreaStyle,
    LineStyle,
    QuantityType,
    SymbolStyle,
    TextStyle,
    XYData,
    XYDataFromFile,
    XYDataFromFreeBody,
    XYDataFromHistory,
    XYDataFromPath,
    XYDataFromShellThickness,
    xyDataListFromField,
)

# inspected from Abaqus cli



from typing_extensions import Literal

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ..UtilityAndView.abaqusConstants import (
    ADD,
    LESS_THAN,
    MAXIMUM,
    MODEL,
    MOVEMENT,
    PREVIOUS,
    SymbolicConstant,
)
from ..UtilityAndView.abaqusConstants import abaqusConstants as C
from .StopCondition import StopCondition


@abaqus_class_doc
class LocalStopCondition(StopCondition):
    """The LocalStopCondition object defines a local stop condition. The LocalStopCondition object is derived
    from the StopCondition object.

    .. note::
        This object can be accessed by::

            import optimization
            mdb.models[name].optimizationTasks[name].stopConditions[name]
    """

    #: A String specifying the stop condition repository key.
    name: str

    #: A Float specifying the factor used to modify the reference value.
    referenceFactor: float

    #: A SymbolicConstant specifying the operation used to compare the selected value to the
    #: reference value. Possible values are LESS_THAN, LESS_THAN_EQUAL, EQUAL,
    #: GREATER_THAN_EQUAL, and GREATER_THAN. The default value is LESS_THAN.
    comparisonOperation: SymbolicConstant = LESS_THAN

    #: A SymbolicConstant specifying the variable identifier of the compared value. Possible
    #: values are:
    #:
    #: - ABSOLUTE_GROWTH_MOVEMENT
    #: - ABSOLUTE_SHRINK_MOVEMENT
    #: - GROWTH_MOVEMENT
    #: - SHRINK_MOVEMENT
    #: - MOVEMENT
    #: - TOTAL_ABSOLUTE_MOVEMENT
    #: - EQUIV_STRESS
    #: - FREE_TASK_REGION_EQUIV_STRESS
    #: - RESTRICTED_TASK_REGION_EQUIV_STRESS
    #: - SURFACE_POINT_EQUIV_STRESS
    #: - TASK_REGION_EQUIV_STRESS
    #:
    #: The default value is MOVEMENT.
    identifier: SymbolicConstant = MOVEMENT

    #: A SymbolicConstant specifying the operation used to evaluate values in the region.
    #: Possible values are MAXIMUM, MINIMUM, and SUM. The default value is MAXIMUM.
    identifierOperation: SymbolicConstant = MAXIMUM

    #: A SymbolicConstant specifying the iteration from which a value is compared to the
    #: reference value. Possible values are FIRST and PREVIOUS. The default value is PREVIOUS.
    referenceDesignCycle: SymbolicConstant = PREVIOUS

    #: A SymbolicConstant specifying the operation used to modify the reference value by the
    #: reference factor. Possible values are ADD, DIVIDE, MULTIPLY, and SUBTRACT. The default
    #: value is ADD.
    referenceOperation: SymbolicConstant = ADD

    #: The SymbolicConstant MODEL or a Region object specifying the region to which the stop
    #: condition is applied. The default value is MODEL.
    region: SymbolicConstant = MODEL

    @abaqus_method_doc
    def __init__(
        self,
        name: str,
        referenceFactor: float,
        comparisonOperation: Literal[
            C.EQUAL, C.GREATER_THAN_EQUAL, C.LESS_THAN_EQUAL, C.LESS_THAN, C.GREATER_THAN
        ] = LESS_THAN,
        identifier: Literal[
            C.TOTAL_ABSOLUTE_MOVEMENT,
            C.GROWTH_MOVEMENT,
            C.SURFACE_POINT_EQUIV_STRESS,
            C.ABSOLUTE_GROWTH_MOVEMENT,
            C.TASK_REGION_EQUIV_STRESS,
            C.SHRINK_MOVEMENT,
            C.RESTRICTED_TASK_REGION_EQUIV_STRESS,
            C.EQUIV_STRESS,
            C.MOVEMENT,
            C.FREE_TASK_REGION_EQUIV_STRESS,
            C.ABSOLUTE_SHRINK_MOVEMENT,
        ] = MOVEMENT,
        identifierOperation: Literal[C.SUM, C.MINIMUM, C.MAXIMUM] = MAXIMUM,
        referenceDesignCycle: Literal[C.PREVIOUS, C.FIRST] = PREVIOUS,
        referenceOperation: Literal[C.MULTIPLY, C.DIVIDE, C.SUBTRACT, C.ADD] = ADD,
        region: Literal[C.MODEL] = MODEL,
    ):
        """This method creates a LocalStopCondition object.

        .. note::
            This function can be accessed by::

                mdb.models[name].optimizationTasks[name].LocalStopCondition

        Parameters
        ----------
        name
            A String specifying the stop condition repository key.
        referenceFactor
            A Float specifying the factor used to modify the reference value.
        comparisonOperation
            A SymbolicConstant specifying the operation used to compare the selected value to the
            reference value. Possible values are LESS_THAN, LESS_THAN_EQUAL, EQUAL,
            GREATER_THAN_EQUAL, and GREATER_THAN. The default value is LESS_THAN.
        identifier
            A SymbolicConstant specifying the variable identifier of the compared value. Possible
            values are:
            - ABSOLUTE_GROWTH_MOVEMENT
            - ABSOLUTE_SHRINK_MOVEMENT
            - GROWTH_MOVEMENT
            - SHRINK_MOVEMENT
            - MOVEMENT
            - TOTAL_ABSOLUTE_MOVEMENT
            - EQUIV_STRESS
            - FREE_TASK_REGION_EQUIV_STRESS
            - RESTRICTED_TASK_REGION_EQUIV_STRESS
            - SURFACE_POINT_EQUIV_STRESS
            - TASK_REGION_EQUIV_STRESS

            The default value is MOVEMENT.
        identifierOperation
            A SymbolicConstant specifying the operation used to evaluate values in the region.
            Possible values are MAXIMUM, MINIMUM, and SUM. The default value is MAXIMUM.
        referenceDesignCycle
            A SymbolicConstant specifying the iteration from which a value is compared to the
            reference value. Possible values are FIRST and PREVIOUS. The default value is PREVIOUS.
        referenceOperation
            A SymbolicConstant specifying the operation used to modify the reference value by the
            reference factor. Possible values are ADD, DIVIDE, MULTIPLY, and SUBTRACT. The default
            value is ADD.
        region
            The SymbolicConstant MODEL or a Region object specifying the region to which the stop
            condition is applied. The default value is MODEL.

        Returns
        -------
        LocalStopCondition
            A LocalStopCondition object.
        """
        super().__init__()

    @abaqus_method_doc
    def setValues(
        self,
        comparisonOperation: Literal[
            C.EQUAL, C.GREATER_THAN_EQUAL, C.LESS_THAN_EQUAL, C.LESS_THAN, C.GREATER_THAN
        ] = LESS_THAN,
        identifier: Literal[
            C.TOTAL_ABSOLUTE_MOVEMENT,
            C.GROWTH_MOVEMENT,
            C.SURFACE_POINT_EQUIV_STRESS,
            C.ABSOLUTE_GROWTH_MOVEMENT,
            C.TASK_REGION_EQUIV_STRESS,
            C.SHRINK_MOVEMENT,
            C.RESTRICTED_TASK_REGION_EQUIV_STRESS,
            C.EQUIV_STRESS,
            C.MOVEMENT,
            C.FREE_TASK_REGION_EQUIV_STRESS,
            C.ABSOLUTE_SHRINK_MOVEMENT,
        ] = MOVEMENT,
        identifierOperation: Literal[C.SUM, C.MINIMUM, C.MAXIMUM] = MAXIMUM,
        referenceDesignCycle: Literal[C.PREVIOUS, C.FIRST] = PREVIOUS,
        referenceOperation: Literal[C.MULTIPLY, C.DIVIDE, C.SUBTRACT, C.ADD] = ADD,
        region: Literal[C.MODEL] = MODEL,
    ):
        """This method modifies the LocalStopCondition object.

        Parameters
        ----------
        comparisonOperation
            A SymbolicConstant specifying the operation used to compare the selected value to the
            reference value. Possible values are LESS_THAN, LESS_THAN_EQUAL, EQUAL,
            GREATER_THAN_EQUAL, and GREATER_THAN. The default value is LESS_THAN.
        identifier
            A SymbolicConstant specifying the variable identifier of the compared value. Possible
            values are:
            - ABSOLUTE_GROWTH_MOVEMENT
            - ABSOLUTE_SHRINK_MOVEMENT
            - GROWTH_MOVEMENT
            - SHRINK_MOVEMENT
            - MOVEMENT
            - TOTAL_ABSOLUTE_MOVEMENT
            - EQUIV_STRESS
            - FREE_TASK_REGION_EQUIV_STRESS
            - RESTRICTED_TASK_REGION_EQUIV_STRESS
            - SURFACE_POINT_EQUIV_STRESS
            - TASK_REGION_EQUIV_STRESS

            The default value is MOVEMENT.
        identifierOperation
            A SymbolicConstant specifying the operation used to evaluate values in the region.
            Possible values are MAXIMUM, MINIMUM, and SUM. The default value is MAXIMUM.
        referenceDesignCycle
            A SymbolicConstant specifying the iteration from which a value is compared to the
            reference value. Possible values are FIRST and PREVIOUS. The default value is PREVIOUS.
        referenceOperation
            A SymbolicConstant specifying the operation used to modify the reference value by the
            reference factor. Possible values are ADD, DIVIDE, MULTIPLY, and SUBTRACT. The default
            value is ADD.
        region
            The SymbolicConstant MODEL or a Region object specifying the region to which the stop
            condition is applied. The default value is MODEL.
        """
        ...



from sphinx.application import Sphinx
from sphinx.ext.autodoc import ALL, ClassDocumenter


class AbqpyClassDocumenter(ClassDocumenter):
    """Toggles the class members section"""

    def document_members(self, all_members: bool = False) -> None:
        # find out all the members that are documented
        want_all = all_members or self.options.inherited_members or self.options.members is ALL
        _, members = self.get_object_members(want_all)
        if members:
            sourcename = self.get_sourcename()
            self.add_line("", sourcename)
            self.add_line(".. rubric:: Member Details:", sourcename)
            self.add_line("", sourcename)
            self.add_line(".. toggle::", sourcename)
            self.add_line("", sourcename)
            self.add_line("    .. placeholder for empty members", sourcename)
            self.add_line("", sourcename)
            self.indent += " " * 4
        super().document_members(all_members)


def setup(app: Sphinx):
    app.setup_extension("sphinx.ext.autodoc")  # Require autodoc extension
    app.setup_extension("sphinx_togglebutton")  # Require sphinx-togglebutton extension
    app.add_autodocumenter(AbqpyClassDocumenter, override=True)
    return {
        "version": "0.0.1",
        "parallel_read_safe": True,
        "parallel_write_safe": True,
    }



from typing import Optional, Tuple

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc


@abaqus_class_doc
class IgnoredEdge:
    """An IgnoredEdge object is a one-dimensional region of geometry that has been abstracted away by a virtual
    topology feature.

    .. note::
        This object can be accessed by::

            import part
            mdb.models[name].parts[name].ignoredEdges[i]
            import assembly
            mdb.models[name].rootAssembly.allInstances[name].ignoredEdges[i]
            mdb.models[name].rootAssembly.instances[name].ignoredEdges[i]
    """

    #: An Int specifying the index of the IgnoredEdge in the IgnoredEdgeArray.
    index: Optional[int] = None

    #: A tuple of Floats specifying the **X**, **Y**, and **Z** coordinates of a point located on
    #: the edge.
    pointOn: Optional[float] = None

    @abaqus_method_doc
    def getSize(self, printResults: str = True):
        """This method returns a Float indicating the length of the edge.

        Parameters
        ----------
        printResults
            A Bool specifying whether verbose output is printed. The default is True.

        Returns
        -------
        float
            A Float.
        """
        ...

    @abaqus_method_doc
    def getRadius(self):
        """This method returns the radius of a circular IgnoredEdge object.

        Returns
        -------
        float
            A Float specifying the radius.

        Raises
        ------
        The given IgnoredEdge object is not circular
        """
        ...

    @abaqus_method_doc
    def getCurvature(self, parameter: float, point: Tuple[float, float, float]):
        """This method returns curvature information at a location on the IgnoredEdge object.

        Parameters
        ----------
        parameter
            A Float specifying the normalized parameter location on the IgnoredEdge where the
            curvature is to be computed. This argument is mutually exclusive with the argument
            **point**.
        point
            A tuple of **X**, **Y**, and **Z** coordinates of a point at which the curvature is to be
            computed. If **point** does not lie on the IgnoredEdge an attempt is made to project it
            onto the IgnoredEdge and use the projected point.

        Returns
        -------
        dict
            A dictionary with keys 'evaluationPoint', 'curvature', 'radius', 'tangent'. Where
            'evaluationPoint' specifies the location at which the curvature was computed.
            'curvature' specifies the curvature vector at that location. 'radius' is the Radius of
            curvature and 'tangent' specifies the tangent to the IgnoredEdge at that location.

        Raises
        ------
        The given IgnoredEdge is straight
        """
        ...



from pathlib import Path

from .cli import AbqpyCLI, abaqus
from .run import run

try:
    from ._version import version as _default_version
except ImportError:
    _default_version = "2021.0.0-dev"


def _get_version():
    """Return the version string used for __version__."""
    # Only shell out to a git subprocess if really needed, and not on a
    # shallow clone, such as those used by CI, as the latter would trigger
    # a warning from setuptools_scm.
    root = Path(__file__).resolve().parents[2]
    if (root / ".git").exists() and not (root / ".git/shallow").exists():
        try:
            import setuptools_scm

            return setuptools_scm.get_version(
                root=str(root),
                version_scheme="guess-next-dev",
                fallback_version=_default_version,
            )
        except Exception:
            return _default_version
    else:  # Get the version from the _version.py setuptools_scm file.
        return _default_version


__version__ = _get_version()
__semver__ = __version__.split("+")[0]



from typing import Optional

from typing_extensions import Literal

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ..UtilityAndView.abaqusConstants import SymbolicConstant
from ..UtilityAndView.abaqusConstants import abaqusConstants as C
from .OdbMeshNode import OdbMeshNode


@abaqus_class_doc
class OdbDatumCsys:
    """The OdbDatumCsys object contains a coordinate system that can be stored in an output database. You can
    create the datum coordinate system in the Visualization module during an Abaqus/CAE session and save the
    datum coordinate system to the output database before you exit Abaqus/CAE. Alternatively, the analysis code
    can write the datum coordinate system to the output database.

    .. note::
        This object can be accessed by::

            import odbAccess
            session.odbs[name].rootAssembly.datumCsyses[name]
    """

    #: A String specifying the repository key.
    name: str = ""

    #: A SymbolicConstant specifying the type of coordinate system. Possible values are
    #: CARTESIAN, CYLINDRICAL, and SPHERICAL.
    coordSysType: Optional[SymbolicConstant] = None

    #: A tuple of Floats specifying the coordinates of the origin of the datum coordinate
    #: system.
    origin: Optional[float] = None

    #: A tuple of Floats specifying a point on the **X** axis.
    xAxis: Optional[float] = None

    #: A tuple of Floats specifying a point on the **Y** axis.
    yAxis: Optional[float] = None

    #: A tuple of Floats specifying a point on the **Z** axis.
    zAxis: Optional[float] = None

    @abaqus_method_doc
    def DatumCsysByThreePoints(
        self,
        name: str,
        coordSysType: Literal[C.CYLINDRICAL, C.CARTESIAN, C.SPHERICAL],
        origin: tuple,
        point1: tuple,
        point2: tuple,
    ):
        """This method creates an OdbDatumCsys object using three points. A datum coordinate system created with
        this method results in a fixed system.

        .. note::
            This function can be accessed by::

                session.odbs[name].rootAssembly.DatumCsysByThreePoints

        Parameters
        ----------
        name
            A String specifying the repository key.
        coordSysType
            A SymbolicConstant specifying the type of coordinate system. Possible values are
            CARTESIAN, CYLINDRICAL, and SPHERICAL.
        origin
            A sequence of Floats specifying the coordinates of the origin of the datum coordinate
            system.
        point1
            A sequence of Floats specifying the coordinates of a point on the local 1- or rr-axis.
        point2
            A sequence of Floats specifying the coordinates of a point in the 1-2 or rr-θθ plane.

        Returns
        -------
        OdbDatumCsys
            An OdbDatumCsys object.
        """
        ...

    @abaqus_method_doc
    def DatumCsysByThreeNodes(
        self,
        name: str,
        coordSysType: Literal[C.CYLINDRICAL, C.CARTESIAN, C.SPHERICAL],
        origin: OdbMeshNode,
        point1: OdbMeshNode,
        point2: OdbMeshNode,
    ):
        """This method creates an OdbDatumCsys object using the coordinates of three OdbMeshNode objects. A
        datum coordinate system created with this method results in a system that follows the position of the
        three nodes. Results, such as those for displacement, are resolved into the orientation of the datum
        coordinate system without regard to the position of its origin. The last three arguments are given in
        the form of an OdbMeshNode object.

        .. note::
            This function can be accessed by::

                session.odbs[name].rootAssembly.DatumCsysByThreePoints

        Parameters
        ----------
        name
            A String specifying the repository key.
        coordSysType
            A SymbolicConstant specifying the type of coordinate system. Possible values are
            CARTESIAN, CYLINDRICAL, and SPHERICAL.
        origin
            An OdbMeshNode object specifying a node at the origin of the datum coordinate system.
        point1
            An OdbMeshNode object specifying a node on the local 1- or rr-axis.
        point2
            An OdbMeshNode object specifying a node in the 1-2 or rr-θθ plane.

        Returns
        -------
        OdbDatumCsys
            An OdbDatumCsys object.
        """
        ...

    @abaqus_method_doc
    def DatumCsysByThreeCircNodes(
        self,
        name: str,
        coordSysType: Literal[C.CYLINDRICAL, C.CARTESIAN, C.SPHERICAL],
        node1Arc: OdbMeshNode,
        node2Arc: OdbMeshNode,
        node3Arc: OdbMeshNode,
    ):
        """This method is convenient to use where there are no nodes along the axis of a hollow cylinder or at
        the center of a hollow sphere. The three nodes that you provide as arguments determine a circle in
        space. The center of the circle is the origin of the datum coordinate system. The normal to the circle
        is parallel to the zz-axis of a cylindrical coordinate system or to the ϕϕ-axis of a spherical
        coordinate system. The line from the origin to the first node defines the rr-axis.

        .. note::
            This function can be accessed by::

                session.odbs[name].rootAssembly.DatumCsysByThreePoints

        Parameters
        ----------
        name
            A String specifying the repository key.
        coordSysType
            A SymbolicConstant specifying the type of coordinate system. Possible values are
            CARTESIAN, CYLINDRICAL, and SPHERICAL.
        node1Arc
            An OdbMeshNode object that lies on the circular arc.
        node2Arc
            An OdbMeshNode object that lies on the circular arc.
        node3Arc
            An OdbMeshNode object that lies on the circular arc.

        Returns
        -------
        OdbDatumCsys
            An OdbDatumCsys object.
        """
        ...

    @abaqus_method_doc
    def DatumCsysBy6dofNode(
        self, name: str, coordSysType: Literal[C.CYLINDRICAL, C.CARTESIAN, C.SPHERICAL], origin: OdbMeshNode
    ):
        """A datum coordinate system created with this method results in a system that follows the position of a
        node. The node location defines the origin of the datum coordinate system. The rotational displacement
        (UR1, UR2, UR3) of the node defines the orientation of the coordinate system axes. Results, such as
        those for displacement, are resolved into the orientation of the datum coordinate system without regard
        to the position of its origin. The last argument is given in the form of an OdbMeshNode object.

        .. note::
            This function can be accessed by::

                session.odbs[name].rootAssembly.DatumCsysByThreePoints

        Parameters
        ----------
        name
            A String specifying the repository key.
        coordSysType
            A SymbolicConstant specifying the type of coordinate system. Possible values are
            CARTESIAN, CYLINDRICAL, and SPHERICAL.
        origin
            An OdbMeshNode object specifying the origin of the datum coordinate system.

        Returns
        -------
        OdbDatumCsys
            An OdbDatumCsys object.
        """
        ...

    @abaqus_method_doc
    def DatumCsys(self, name: str, datumCsys: "OdbDatumCsys"):
        """This method copies oneOdbDatumCsys object to a new OdbDatumCsys object.

        .. note::
            This function can be accessed by::

                session.odbs[name].rootAssembly.DatumCsysByThreePoints

        Parameters
        ----------
        name
            A String specifying the repository key.
        datumCsys
            An OdbDatumCsys object specifying the object to be copied.

        Returns
        -------
        OdbDatumCsys
            An OdbDatumCsys object.
        """
        ...



from os import environ as _environ

import abaqus
import assembly
import connector
import connectorBehavior
import displayGroupMdbToolset as dgm
import displayGroupOdbToolset as dgo
import interaction
import job
import load
import material
import mesh
import optimization
import part
import regionToolset
import section
import sketch
import step
import visualization
import xyPlot

# if 'HKS_WITH_STUB' in _environ:
#     import stub



from typing import Optional

from abqpy.decorators import abaqus_class_doc

from ..UtilityAndView.abaqusConstants import SymbolicConstant
from .InteractionState import InteractionState


@abaqus_class_doc
class ActuatorSensorState(InteractionState):
    """The ActuatorSensorState object stores the propagating data of an actuator sensor in a step. One instance
    of this object is created internally by the ActuatorSensor object for each step. The instance is also
    deleted internally by the ActuatorSensor object. The ActuatorSensorState object has no constructor, methods,
    or members. The ActuatorSensorState object is derived from the InteractionState object.

    .. note::
        This object can be accessed by::

            import interaction
            mdb.models[name].steps[name].interactionStates[name]
    """

    #: A SymbolicConstant specifying the propagation state of the InteractionState object.
    #: Possible values are:
    #:
    #: - NOT_YET_ACTIVE
    #: - CREATED
    #: - PROPAGATED
    #: - MODIFIED
    #: - DEACTIVATED
    #: - NO_LONGER_ACTIVE
    #: - TYPE_NOT_APPLICABLE
    #: - INSTANCE_NOT_APPLICABLE
    #: - BUILT_INTO_BASE_STATE
    status: Optional[SymbolicConstant] = None



from typing import Optional

from abqpy.decorators import abaqus_class_doc

from ..UtilityAndView.abaqusConstants import OFF, Boolean


@abaqus_class_doc
class SelectedProbeValues:
    """The SelectedProbeValues object has no constructor. The SelectedProbeValues object is created when you
    import the Visualization module.

    .. note::
        This object can be accessed by::

            import visualization
            session.selectedProbeValues
    """

    #: An Int specifying the length of the **values** member.
    length: Optional[int] = None

    #: A Boolean specifying whether any probe values have been selected (as is necessary prior
    #: to writing to a file).
    fieldOutputAvailable: Boolean = OFF

    #: A tuple of tuples of Floats specifying the selected probe values.
    values: Optional[float] = None

    #: A tuple of Floats specifying the last sequence of the **values** member.
    lastValues: tuple = ()



from math import *

from abqpy import run

run(cae=False)

from abaqus.Odb.OdbCommands import *
from abaqus.UtilityAndView.BackwardCompatibility import BackwardCompatibility
from abaqusConstants import *

backwardCompatibility = BackwardCompatibility()



from typing import Optional

from abqpy.decorators import abaqus_class_doc

from .Section import Section
from .TransverseShearShell import TransverseShearShell


@abaqus_class_doc
class ShellSection(Section):
    """The ShellSection object defines the properties of a shell section. The ShellSection object is derived
    from the Section object. The ShellSection object has no explicit constructor and no methods or members. The
    ShellSection object is derived from the Section object.

    .. note::
        This object can be accessed by::

            import section
            mdb.models[name].sections[name]
            import odbSection
            session.odbs[name].sections[name]
    """

    #: A String specifying the repository key.
    name: str = ""

    #: A TransverseShearShell object specifying the transverse shear stiffness properties.
    transverseShear: Optional[TransverseShearShell] = None



from typing import Dict, Optional, Sequence, overload

from typing_extensions import Literal

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ..Property.MaterialOrientationArray import MaterialOrientationArray
from ..Property.SectionAssignmentArray import SectionAssignmentArray
from ..UtilityAndView.abaqusConstants import AXIS_1, STACK_3
from ..UtilityAndView.abaqusConstants import abaqusConstants as C
from .AnalyticSurface import AnalyticSurface
from .AnalyticSurfaceSegment import AnalyticSurfaceSegment
from .BeamOrientationArray import BeamOrientationArray
from .OdbDatumCsys import OdbDatumCsys
from .OdbMeshElementArray import OdbMeshElementArray
from .OdbMeshNodeArray import OdbMeshNodeArray
from .OdbRigidBodyArray import OdbRigidBodyArray
from .OdbSet import OdbSet
from .RebarOrientationArray import RebarOrientationArray
from .SectionCategory import SectionCategory


@abaqus_class_doc
class OdbPartBase:
    """The OdbPart object is similar to the kernel Part object and contains nodes and elements, but not
    geometry.

    .. note::
        This object can be accessed by::

            import odbAccess
            session.odbs[name].parts[name]
    """

    #: An OdbMeshNodeArray object.
    nodes: OdbMeshNodeArray = []

    #: An OdbMeshElementArray object.
    elements: OdbMeshElementArray = []

    #: A repository of OdbSet objects specifying node sets.
    nodeSets: Dict[str, OdbSet] = {}

    #: A repository of OdbSet objects specifying element sets.
    elementSets: Dict[str, OdbSet] = {}

    #: A repository of OdbSet objects specifying surfaces.
    surfaces: Dict[str, OdbSet] = {}

    #: A SectionAssignmentArray object.
    sectionAssignments: SectionAssignmentArray = []

    #: A BeamOrientationArray object.
    beamOrientations: BeamOrientationArray = []

    #: A MaterialOrientationArray object.
    materialOrientations: MaterialOrientationArray = []

    #: A RebarOrientationArray object.
    rebarOrientations: RebarOrientationArray = []

    #: An OdbRigidBodyArray object.
    rigidBodies: OdbRigidBodyArray = []

    #: An AnalyticSurface object specifying analytic Surface defined on the instance.
    analyticSurface: AnalyticSurface = AnalyticSurface()

    @abaqus_method_doc
    def __init__(
        self,
        name: str,
        embeddedSpace: Literal[C.THREE_D, C.AXISYMMETRIC, C.TWO_D_PLANAR],
        type: Literal[C.DEFORMABLE_BODY, C.ANALYTIC_RIGID_SURFACE],
    ):
        """This method creates an OdbPart object. Nodes and elements are added to this object at a later stage.

        .. note::
            This function can be accessed by::

                session.odbs[name].Part

        Parameters
        ----------
        name
            A String specifying the part name.
        embeddedSpace
            A SymbolicConstant specifying the dimensionality of the Part object. Possible values are
            THREE_D, TWO_D_PLANAR, and AXISYMMETRIC.
        type
            A SymbolicConstant specifying the type of the Part object. Possible values are
            DEFORMABLE_BODY and ANALYTIC_RIGID_SURFACE.

        Returns
        -------
        OdbPart
            An OdbPart object.
        """
        ...

    @overload
    @abaqus_method_doc
    def addElements(
        self,
        labels: tuple,
        connectivity: tuple,
        type: str,
        elementSetName: str = "",
        sectionCategory: Optional[SectionCategory] = None,
    ):
        """This method adds elements to an OdbPart object using element labels and nodal connectivity.
        Warning:Adding elements not in ascending order of their labels may cause Abaqus/Viewer to plot contours
        incorrectly.

        Parameters
        ----------
        labels
            A sequence of Ints specifying the element labels.
        connectivity
            A sequence of sequences of Ints specifying the nodal connectivity.
        type
            A String specifying the element type.
        elementSetName
            A String specifying a name for this element set. The default value is the empty string.
        sectionCategory
            A SectionCategory object for this element set.
        """
        ...

    @overload
    @abaqus_method_doc
    def addElements(
        self,
        elementData: tuple,
        type: str,
        elementSetName: Optional[str] = None,
        sectionCategory: Optional[SectionCategory] = None,
    ):
        """This method adds elements to an OdbPart object using a sequence of element labels and nodal
        connectivity. Warning:Adding elements not in ascending order of their labels may cause Abaqus/Viewer to
        plot contours incorrectly.

        Parameters
        ----------
        elementData
            A sequence of sequences of Ints specifying the element labels and nodal connectivity, in
            the form ((*label*, **c1**, **c2**, **c3**, **c4**), (*label*, **c1**, **c2**, **c3**, **c4**), ...).
        type
            A String specifying the element type. The value can be user defined.
        elementSetName
            A String specifying a name for this element set. The default value is None.
        sectionCategory
            A SectionCategory object for this element set.
        """
        ...

    @abaqus_method_doc
    def addElements(self, *args, **kwargs):
        ...

    @overload
    def addNodes(self, labels: tuple, coordinates: tuple, nodeSetName: Optional[str] = None):
        """This method adds nodes to an OdbPart object using node labels and coordinates. Warning:Adding nodes
        not in ascending order of their labels may cause Abaqus/Viewer to plot contours incorrectly.

        Parameters
        ----------
        labels
            A sequence of Ints specifying the node labels.
        coordinates
            A sequence of sequences of Floats specifying the nodal coordinates.
        nodeSetName
            A String specifying a name for this node set. The default value is None.
        """
        ...

    @overload
    @abaqus_method_doc
    def addNodes(self, nodeData: tuple, nodeSetName: Optional[str] = None):
        """This method adds nodes to an OdbPart object using a sequence of node labels and coordinates.
        Warning:Adding nodes not in ascending order of their labels may cause Abaqus/Viewer to plot contours
        incorrectly.

        Parameters
        ----------
        nodeData
            A sequence of tuples specifying the node labels and coordinates, in the form ((*label*,
            **x**, **y**, **z**), (*label*, **x**, **y**, **z**), ...).
        nodeSetName
            A String specifying a name for this node set. The default value is None.
        """
        ...

    @abaqus_method_doc
    def addNodes(self, *args, **kwargs):
        ...

    def assignBeamOrientation(self, region: str, method: Literal[C.N1_COSINES], vector: tuple):
        """This method assigns a beam section orientation to a region of a part instance.

        Parameters
        ----------
        region
            An OdbSet specifying a region on an instance.
        method
            A SymbolicConstant specifying the assignment method. Only a value of N1_COSINES is
            currently supported.
        vector
            A sequence of three Floats specifying the approximate local  n1n1 -direction of the beam
            cross-section.
        """
        ...

    @abaqus_method_doc
    def assignMaterialOrientation(
        self,
        region: str,
        localCSys: OdbDatumCsys,
        axis: Literal[C.AXIS_1, C.AXIS_3, C.AXIS_2] = AXIS_1,
        angle: float = 0,
        stackDirection: Literal[C.STACK_2, C.STACK_ORIENTATION, C.STACK_3, C.STACK_1] = STACK_3,
    ):
        """This method assigns a material orientation to a region of a part instance.

        Parameters
        ----------
        region
            An OdbSet specifying a region on an instance.
        localCSys
            An OdbDatumCsys object specifying the local coordinate system or None, indicating the
            global coordinate system.
        axis
            A SymbolicConstant specifying the axis of a cylindrical or spherical datum coordinate
            system about which an additional rotation is applied. For shells this axis is also the
            shell normal. Possible values are AXIS_1, AXIS_2, and AXIS_3. The default value is
            AXIS_1.
        angle
            A Float specifying the angle of the additional rotation. The default value is 0.0.
        stackDirection
            A SymbolicConstant specifying the stack or thickness direction of the material. Possible
            values are STACK_1, STACK_2, STACK_3, and STACK_ORIENTATION. The default value is
            STACK_3.
        """
        ...

    @abaqus_method_doc
    def assignRebarOrientation(
        self,
        region: str,
        localCsys: OdbDatumCsys,
        axis: Literal[C.AXIS_1, C.AXIS_3, C.AXIS_2] = AXIS_1,
        angle: float = 0,
    ):
        """This method assigns a rebar reference orientation to a region of a part instance.

        Parameters
        ----------
        region
            An OdbSet specifying a region on an instance.
        localCsys
            An OdbDatumCsys object specifying the local coordinate system or None, indicating the
            global coordinate system.
        axis
            A SymbolicConstant specifying the axis of a cylindrical or spherical datum coordinate
            system about which an additional rotation is applied. For shells this axis is also the
            shell normal. Possible values are AXIS_1, AXIS_2, and AXIS_3. The default value is
            AXIS_1.
        angle
            A Float specifying the angle of the additional rotation. The default value is 0.0.
        """
        ...

    @abaqus_method_doc
    def getElementFromLabel(self, label: int):
        """This method is used to retrieved an element with a specific label from a part object.

        Parameters
        ----------
        label
            An Int specifying the element label.

        Returns
        -------
        OdbMeshElement
            An OdbMeshElement object.

        Raises
        ------
        OdbError
            Invalid element label, If no element with the specified label exists.
        """
        ...

    @abaqus_method_doc
    def getNodeFromLabel(self, label: int):
        """This method is used to retrieved a node with a specific label from a part object.

        Parameters
        ----------
        label
            An Int specifying the node label.

        Returns
        -------
        OdbMeshNode
            An OdbMeshNode object.

        Raises
        ------
        OdbError
            Invalid node label, If no node with the specified label exists.
        """
        ...

    @abaqus_method_doc
    def AnalyticRigidSurf2DPlanar(self, name: str, profile: Sequence[AnalyticSurfaceSegment], filletRadius: str = 0):
        """This method is used to define a two-dimensional AnalyticSurface object on the part object.

        Parameters
        ----------
        name
            The name of the analytic surface.
        profile
            A sequence of AnalyticSurfaceSegment objects or an OdbSequenceAnalyticSurfaceSegment
            object.
        filletRadius
            A Double specifying the radius of curvature to smooth discontinuities between adjoining
            segments. The default value is 0.0.

        Raises
        ------
        OdbError: 2D-Planar Analytic Rigid Surface can be defined only if the part is of type
        TWO_D_PLANAR or AXISYMMETRIC
            If OdbPart is of type THREE_D.
        """
        ...

    @abaqus_method_doc
    def AnalyticRigidSurfExtrude(self, name: str, profile: Sequence[AnalyticSurfaceSegment], filletRadius: str = 0):
        """This method is used to define a three-dimensional cylindrical AnalyticSurface on the part object.

        Parameters
        ----------
        name
            The name of the analytic surface.
        profile
            A sequence of AnalyticSurfaceSegment objects or an OdbSequenceAnalyticSurfaceSegment
            object.
        filletRadius
            A Double specifying the radius of curvature to smooth discontinuities between adjoining
            segments. The default value is 0.0.

        Raises
        ------
        OdbError: Analytic Rigid Surface of type CYLINDER can be defined only if the part is
        of type THREE_D
            If OdbPart is not of type THREE_D.
        """
        ...

    @abaqus_method_doc
    def AnalyticRigidSurfRevolve(self, name: str, profile: Sequence[AnalyticSurfaceSegment], filletRadius: str = 0):
        """This method is used to define a three-dimensional AnalyticSurface of revolution on the part object.

        Parameters
        ----------
        name
            The name of the analytic surface.
        profile
            A sequence of AnalyticSurfaceSegment objects or an OdbSequenceAnalyticSurfaceSegment
            object.
        filletRadius
            A Double specifying the radius of curvature to smooth discontinuities between adjoining
            segments. The default value is 0.0.

        Raises
        ------
        OdbError: Analytic Rigid Surface of type REVOLUTION can be defined only if the part is
        of type THREE_D
            If OdbPart is not of type THREE_D.
        """
        ...
